@@ -3,13 +3,15 @@ use strict;
 use subs qw(_common_format _checksum is_valid_checksum);
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $debug %country_data);
 
+use Exporter;
+
 my $debug = 0;
 
 @ISA       = qw(Exporter);
 @EXPORT    = qw();
-@EXPORT_OK = qw(is_valid_checksum);
+@EXPORT_OK = qw(is_valid_checksum ean_to_isbn isbn_to_ean);
 
-$VERSION   = '19980329';
+$VERSION   = '1.5';
 
 sub new
 	{
@@ -51,7 +53,7 @@ sub new
 	PUBLISHER_CODE:
 	while( defined( $trial_publisher_code = substr($self->{'isbn'}, 
 	                                      $country_code_length,
-	                                      $count++) )  and
+	                                      $count) )  and
 	                                      $count < $max_publisher_code_length)
 		{
 		my $trial_publisher_code_length = length $trial_publisher_code;
@@ -62,8 +64,8 @@ sub new
 		PAIR: 
 		while( @pairs )
 			{
-			my $lower_bound = shift @pairs;
-			my $upper_bound = shift @pairs;
+			my $lower_bound  = shift @pairs;
+			my $upper_bound  = shift @pairs;
 			my $lower_length = length $lower_bound;
 			my $upper_length = length $upper_bound;
 			
@@ -92,10 +94,24 @@ sub new
 					$trial_publisher_code_length;
 			    last PUBLISHER_CODE;
 			    }
-			}  	
+			} 
+			
+		$count++; 	
 		
 		}
 
+	#get the book code, which is everything between the
+	#publisher code the and the checksum
+	$common_data =~ m/  
+					$self->{'country_code'} 
+					$self->{'publisher_code'}
+					(.+)
+					([\dxX])
+					$
+					/x;
+	$self->{'article_code'} = $1 if defined $1 and defined $2;
+	$self->{'checksum'}     = $2 if defined $1 and defined $2;
+
 	if( is_valid_checksum $self->{'isbn'} and 
 	    defined $self->{'country_code'}  and 
 	    defined $self->{'publisher_code'} )
@@ -117,21 +133,12 @@ sub new
 
 sub isbn             { my $self = shift; return $self->{'isbn'} }
 sub is_valid         { my $self = shift; return $self->{'valid'} }
-sub publisher_code   { my $self = shift; return $self->{'publisher_code'} }
 sub country_code     { my $self = shift; return $self->{'country_code'} }
-sub country          { my $self = shift; return $self->{'country'} }
+sub publisher_code   { my $self = shift; return $self->{'publisher_code'} }
+sub article_code     { my $self = shift; return $self->{'article_code'} }
+sub checksum         { my $self = shift; return $self->{'checksum'} }
 sub hyphen_positions { my $self = shift; return @{$self->{'positions'}} }
 
-# the only exportable function
-#
-sub is_valid_checksum
-	{
-	my $data = _common_format shift;
-	
-	return 1 if substr($data, 9, 1) eq _checksum $data;
-	
-	return 0;
-	}
 	
 sub fix_checksum
 	{
@@ -160,7 +167,7 @@ sub as_string
 	return undef unless $self->is_valid;
 	my $isbn = $self->isbn;
 	
-	foreach my $position ( sort {$b <=> $a} @$array_ref )
+	foreach my $position ( sort { $b <=> $a } @$array_ref )
 		{
 		next if $position > 9 or $position < 1;
 		substr($isbn, $position, 0) = '-';
@@ -168,12 +175,75 @@ sub as_string
 			
 	return $isbn;
 	}
+	
+sub as_ean
+	{
+	my $self = shift;
+	
+	my $isbn = ref $self ? $self->as_string([]) : _common_format $self;
+	
+	return unless length $isbn == 10;
+	
+	my $ean = '978' . substr($isbn, 0, 9);;
+	
+	my $sum = 0;
+	foreach my $index ( 0, 2, 4, 6, 8, 10 )
+		{
+		$sum +=     substr($ean, $index, 1);
+		$sum += 3 * substr($ean, $index + 1, 1);
+		}
+	
+	#take the next higher multiple of 10 and subtract the sum.
+	#if $sum is 37, the next highest multiple of ten is 40. the
+	#check digit would be 40 - 37 => 3.
+	$ean .= ( 10 * ( int( $sum / 10 ) + 1 ) - $sum ) % 10;
+	
+	return $ean;
+	}
+	
+sub is_valid_checksum
+	{
+	my $data = _common_format shift;
+	
+	return 1 if substr($data, 9, 1) eq _checksum $data;
+	
+	return 0;
+	}
 
+sub ean_to_isbn
+	{
+	my $ean = shift;
+	
+	$ean =~ s/[^0-9]//g;
+	
+	return undef unless length $ean == 13;
+	return undef unless substr($ean, 0, 3) eq '978';
+		
+	my $isbn = new Business::ISBN( substr($ean, 3, 9) . '1' );
+	
+	$isbn->fix_checksum;
+	
+	return $isbn->as_string([]) if $isbn->is_valid;
+	
+	return undef;
+	}
+
+		
+sub isbn_to_ean
+	{
+	my $isbn = _common_format shift;
+	
+	return undef unless is_valid_checksum($isbn);
+	
+	return as_ean($isbn);
+	}	
+	
+#internal function.  you don't get to use this one.
 sub _check_validity
 	{
 	my $self = shift;
 	
-	if( is_valid_checksum and defined $self->{'country_code'}
+	if( is_valid_checksum $self->{'isbn'} and defined $self->{'country_code'}
 	    and defined $self->{'publisher_code'} )
 	    {
 	    $self->{'valid'} = 1;
@@ -182,10 +252,10 @@ sub _check_validity
 		{
 		$self->{'valid'} = -3 if not defined $self->{'publisher_code'};
 		$self->{'valid'} = -2 if not defined $self->{'country_code'};
-		$self->{'valid'} = -1 if not is_valid_checksum;
+		$self->{'valid'} = -1 if not is_valid_checksum $self->{'isbn'};
 		}
 	}
-	
+
 #internal function.  you don't get to use this one.
 sub _checksum
 	{
@@ -355,9 +425,8 @@ Business::ISBN - work with International Standard Book Numbers
 	#this not does affect the default positions
 	print $isbn_object->as_string([]);
 	
-	#print the country code, country, or publisher code
+	#print the country code or publisher code
 	print $isbn->country_code;
-	print $isbn->country;
 	print $isbn->publisher_code;
 	
 	#check to see if the ISBN is valid
@@ -367,6 +436,19 @@ Business::ISBN - work with International Standard Book Numbers
 	#in the checksum!
 	$isbn_object->fix_checksum;
 
+	#EXPORTABLE FUNCTIONS
+	
+	use Business::ISBN qw( is_valid_checksum isbn_to_ean ean_to_isbn );
+	
+	#verify the checksum
+	if( is_valid_checksum('0123456789') ) { ... }
+	
+	#convert to EAN (European Article Number)
+	$ean = isbn_to_ean('1565921496');
+
+	#convert from EAN (European Article Number)
+	$isbn = ean_to_isbn('9781565921498');
+
 =head1 DESCRIPTION
 
 =head2 new($isbn)
@@ -379,10 +461,10 @@ internal representation.  The resulting string must look
 like an ISBN - the first nine characters must be digits and
 the tenth character must be a digit, 'x', or 'X'.
 
-The constructor attempts to determine the country code
-and the publisher code based on data from .  If these
-data cannot be determined, the constructor returns an
-error number:
+The constructor attempts to determine the country
+code and the publisher code.  If these data cannot
+be determined, the constructor returns an error
+number:
 
 	-3 Could not determine publisher code
 	-2 Could not determine country code
@@ -394,7 +476,10 @@ use the C<fix_checksum()> method.  Despite the disclaimer in
 the discussion of that method, the author has found it
 extremely useful.  One should check the validity of the ISBN
 with C<is_valid()> rather than relying on the return value
-of the constructor.
+of the constructor.  If all one wants to do is check the
+validity of an ISBN, one can skip the object-oriented 
+interface and use the c<is_valid_checksum()> function
+which is exportable on demand.
 
 If the constructor decides it can't create an object, it
 returns undef.  It may do this if the string passed as the
@@ -408,10 +493,6 @@ Returns the publisher code.
 
 Returns the country code.
 
-=head2 $obj->country
-
-Returns the country designation associated with the country code.
-
 =head2 $obj->hyphen_positions
 
 Returns the list of hyphen positions as determined from the 
@@ -440,8 +521,8 @@ A terminating 'x' is changed to 'X'.
 Returns 1 if the checksum is valid and the country and
 publisher codes are defined.
 
-Returns -1 if the ISBN does not pass the checksum test, and 1
-if it does.  The constructor accepts invalid ISBN's so that
+Returns -1 if the ISBN does not pass the checksum test.  
+The constructor accepts invalid ISBN's so that
 they might be fixed with C<fix_checksum>.  
 
 Returns -2 if a country code could not be determined (relies
@@ -460,16 +541,48 @@ at all.  It only produces a string that passes the checksum
 routine.  If the ISBN passed to the constructor was invalid,
 the error might have been in any of the other nine positions.
 
+=head2  $obj->as_ean()
+
+Converts the ISBN to the equivalent EAN (European Article Number).
+No pricing extension is added.  Returns the EAN as a string.  This
+method can also be used as an exportable function since it checks
+its argument list to determine what to do.
+
+=head1 EXPORTABLE FUNCTIONS
+
+Some functions can be used without the object interface.  These
+do not use object technology behind the scenes.
+
+=head2 is_valid_checksum('1565921496')
+
+Takes the ISBN string and runs it through the checksum
+comparison routine.  Returns 1 if the ISBN is valid, 0 otherwise.
+
+=head2 isbn_to_ean('1565921496')
+
+Takes the ISBN string and converts it to the equivalent
+EAN string.  This function checks for a valid ISBN and will return
+undef for invalid ISBNs, otherwise it returns the EAN as a string.
+Uses as_ean internally, which checks its arguments to determine
+what to do.
+
+=head2 ean_to_isbn('9781565921498')
+
+Takes the EAN string and converts it to the equivalent
+ISBN string.  This function checks for a valid ISBN and will return
+undef for invalid ISBNs, otherwise it returns the EAN as a string.
+Uses as_ean internally, which checks its arguments to determine
+what to do.
+
 =head1 AUTHOR
 
-brian d foy <comdog@computerdog.com>
+brian d foy <brian@smithrenaud.com>
 
-please see <URL:http://computerdog.com/brian/style.html> for 
-guidelines on proper attribution.
+Copyright 2000, Smith Renaud, Inc. 
 
 The coding of this module was supported by Smith Renaud, Inc. 
-<URL:http://www.smithrenaud.com> and released under the terms of 
-the Perl Artistic License.
+<URL:http://www.smithrenaud.com>.  This module is released under 
+the terms of the Perl Artistic License.
 
 Country code and publisher code graciously provided by Steve
 Fisher <stevef@teleord.co.uk> of Whitaker (the UK ISBN folks
@@ -479,4 +592,6 @@ and the major bibliographic data provider in the UK).
 Thanks to Julie Koo of Kaya Publishing <URL:http://www.kaya.com>
 for useful discussions.
 
+Thanks to Mark W. Eichin <eichin@thok.org> for suggestions and
+discussions on EAN support.
 =cut
